import React, { useState, useEffect } from 'react';
import domReady from '@wordpress/dom-ready';
import { createRoot } from '@wordpress/element';
import {  Panel, PanelBody, PanelRow, Button, Spinner } from '@wordpress/components';

import Table from './admin/EditableTable';

const AdminApp = () => {
    const [isLoading, setIsLoading] = useState(false);
    const [isSaving, setIsSaving] = useState(false);
    const [isFetchingNext, setIsFetchingNext] = useState(false);
    const [isFetchingPrev, setIsFetchingPrev] = useState(false);
    const [data, setData] = useState([]);
    const [columns, setColumns] = useState([]);
    const [currentPage, setCurrentPage] = useState(1);
    const [totalPages, setTotalPages] = useState(1);
    const rowsPerPage = 10;

    const fetchData = (page) => {
       
        setIsLoading(true);

        const requestData = { 
            security: wp_ajax.nonce,
            page: page,
            per_page: rowsPerPage
        };

        return fetch(wp_ajax.getdata, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': wp_ajax.nonce,
            },
            body: JSON.stringify(requestData)
        })
        .then(response => response.json())
        .then(responseData => {
            console.log('Fetched Data:', responseData);
            setData(responseData.data);
            setTotalPages(responseData.total_pages);
            setColumns(responseData.header);
            setIsFetchingNext(false);
            setIsFetchingPrev(false);
            setIsLoading(false);
        })
        .catch(error => {
            console.error('Error fetching data:', error);
        });
    };

    useEffect(() => {
        fetchData(currentPage);
    }, [currentPage]);

    

    const [modifiedCells, setModifiedCells] = useState([]);

    const updateMyData = (rowIndex, columnId, value) => {
        setData(oldData =>
            oldData.map((row, index) => {
                if (index === rowIndex) {
                    setModifiedCells(prev => [
                        ...prev.filter(cell => !(cell.rowIndex === rowIndex && cell.columnId === columnId)),
                        { rowIndex, id: row.id, columnId, value },
                    ]);
                    return {
                        ...oldData[rowIndex],
                        [columnId]: value,
                    };
                }
                return row;
            })
        );
    };

    const handleSave = () => {
        setIsSaving(true);

        const requestData = { 
            security: wp_ajax.nonce,
            modifiedCells: modifiedCells
        };

        fetch(wp_ajax.savedata, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': wp_ajax.nonce,
            },
            body: JSON.stringify(requestData)
        })
        .then(response => response.json())
        .then(() => {
            setIsSaving(false);
            alert('Data saved successfully!');
            setModifiedCells([]);
        })
        .catch(error => {
            setIsSaving(false);
            console.error('Error saving data:', error);
        });
    };

    const goToNextPage = () => {
        setIsFetchingNext(true);
        setCurrentPage(prev => Math.min(prev + 1, totalPages));
   
    };

    const goToPreviousPage = () => {
        setIsFetchingPrev(true);
        setCurrentPage(prev => Math.max(prev - 1, 1));
        
    };

    return (
        <Panel header="Edit Product Data">
            <PanelBody>

            {/* Show content loader spinner */}
            {isLoading ? <Spinner /> : <Table columns={columns} data={data} updateMyData={updateMyData} />}
            

            <div style={{ display: 'flex', justifyContent: 'space-between', marginTop: '20px', marginBottom: '20px' }}>
                <Button isPrimary onClick={goToPreviousPage} disabled={currentPage === 1 || isFetchingPrev}>
                    {isFetchingPrev ? <Spinner /> : 'Previous'}
                </Button>
                <span>Page {currentPage} of {totalPages}</span>
                <Button isPrimary onClick={goToNextPage} disabled={currentPage === totalPages || isFetchingNext}>
                    {isFetchingNext ? <Spinner /> : 'Next'}
                </Button>
            </div>

            <Button isPrimary onClick={handleSave} disabled={isSaving}>
                {isSaving ? <Spinner /> : 'Save Settings'}
            </Button>
            </PanelBody>
        </Panel>
    );
};

domReady(() => {
    const root = createRoot(document.getElementById('gmbpu-admin-root'));
    root.render(<AdminApp />);
});

export default AdminApp;
